%% Introduction
% Recently, I read an
% <http://www.wired.com/2014/11/whats-boarding-airplanes-takes-forever article>
% in Wired (TM) magazine that posed a question that I am sure many of you have
% confronted -- what is the most efficient way to get passengers onto an
% airplane. While this article highlights a theoretical (and deterministic)
% approach to doing this, I set about trying to model this situation and try
% some variations of this strategy and others that capture at least a few more
% elements of randomness in the boarding process.


%% Generating a queue of passengers 
% I used MATLAB to generate a queue of passengers utilizing many different
% boarding strategies. These strategies are shown in the figure below. The most
% common one used by airlines is the one labeled 'backToFrontZone'. A colored
% circle in each plot is more bright than another when that passenger has a
% higher probability of being in the passenger queue earlier. I use the
% 'datasample' function in MATLAB to generate a sample passenger queue with
% those weighted probabilities. Each decrease in shade of green represents a 10
% fold decrease in probability of a darker shade dot (i.e. passenger) jumping
% ahead of the brighter shade dot in the queue.

close all; clear all

% Boarding types dictate how you want to order passengers for boarding
boardingTypes = {'frontToBackZone', 'backToFrontZone', 'random', ...
                 'alternateRowZone', 'windowToAisleZone', 'windowToAisleAlternateZone'};
%boardingTypes = {'windowToAisleAlternateZone'};

numRows = 20; % Number of rows in the plane
numSeatsPerRow = 6; % Number of seats per row
crossZoneFactor = 10; % Factor that controls how likely people from different
                      % zones mix. Higher numbers means mixing less likely
expRndMeanStowTime = 1/4; % Mean time (mins) for stowing bags
expRndMeanGetUpSitTime = 1/6; % Mean time (mins) for getting up and sitting down
                              % when passenger needs to let someone into
                              % their row
seatOrders = [];
idealOrders = [];
for k = 1:length(boardingTypes)
    fprintf('Generating %s\n', boardingTypes{k});
    
    % Use MATLAB to produce the passenger order
    [seatOrder, idealOrder] = airseatSetupSim(boardingTypes{k}, 'repeatable', ...
                                numRows, numSeatsPerRow, ...
                                crossZoneFactor, expRndMeanStowTime, ...
                                expRndMeanGetUpSitTime);
    seatOrders = [seatOrders, seatOrder]; %#ok
    idealOrders = [idealOrders, idealOrder]; %#ok
end
airseatDrawBoardingSchemes(boardingTypes, seatOrders);


%% Run simulations
% To model the aircraft cabin, I used a model built in SimEvents.  SimEvents is
% discrete-event simulation library for the Simulink platform. Using this
% library I can model one row of seats and then repeat that row as many times as
% I want to form the cabin. This is shown below.
airseat;

% The stream of passengers are represented as entities (distinct objects) in
% SimEvents and these can be generated to match the order produced in the
% previous step in MATLAB.  Attributes on these entities track the row number
% and seat number of the corresponding passenger. Additionally, one can
% associate random amounts of time that each passenger will take to stow bags
% and sit down. 
%
% Each row of the cabin is modeled using blocks from the library. The aisle
% space adjacent to each row of the cabin is modeled as a Server -- i.e. a space
% that be occupied by one passenger. A passenger occupies that space when they
% are (a) waiting for the cabin ahead to clear up, (b) when they are stowing
% their bags, or (c) waiting for passengers who may be blocking their seat
% access to make room. Even if the cabin is clear up ahead, they take some time
% in that space to model the transit time through the cabin.
%
open_system('airseat/Row1');

% We can now simulate the boarding order and capture the total time it takes to
% fill the cabin. I have also implemented an animation of the cabin using new
% APIs available in R2015a to access each entity and its states. It makes for a
% fun interactive way to visualize each scheme.
% (Note that to make the evaluation "fair" across all strategies, a passenger
% sitting in a particular seat always takes the same amount of time to stow
% luggage and sit down (or get up and give way to someone who they are blocking). 
%
seatingTimeResultsRnd = zeros(1, length(boardingTypes));
seatingTimeResultsIdeal = zeros(1, length(boardingTypes));
figHdl = [];
for k = 1:length(boardingTypes)
    fprintf('Running %s\n', boardingTypes{k});
    [sTime,figHdl] = airseatRunSim('airseat', idealOrders(k), figHdl, ...
                                   numRows, numSeatsPerRow);
    seatingTimeResultsIdeal(k) = sTime;
    fprintf('  Last seating time (ideal): %f\n', sTime);

    [sTime,figHdl] = airseatRunSim('airseat', seatOrders(k), figHdl, ...
                                   numRows, numSeatsPerRow);
    seatingTimeResultsRnd(k) = sTime;
    fprintf('  Last seating time (random): %f\n', sTime);
end
save airseatResults boardingTypes seatOrders idealOrders seatingTimeResultsIdeal seatingTimeResultsRnd

%% Results
% I plotted the results for each scheme. Additionally, I also plotted as a
% dashed line the "ideal" case for each situation. I define ideal as a perfectly
% behaved group that boards exactly in the order prescribed by that scheme with
% no randomness.
%
% The results do validate that the current process used by airlines can only get
% worse if they decide to board front to back! I was surprised that some schemes
% work better than the truly random scheme -- but perhaps I should have expected
% that given the article I started with. It looks like even with the randomness
% thrown intp the determinsitic model presented above -- represented by the
% scheme labeled 'windowToAisleAlternateZone', it performed the best. Maybe when
% I am in the airport next time things would have changed for the better?
%
figure(2); clf
plot(seatingTimeResultsRnd)
hold on
plot(seatingTimeResultsIdeal, '--')
set(gca, 'xtick', 1:length(boardingTypes))
set(gca, 'xticklabel', boardingTypes)
set(gca,'xticklabelrotation',90)

