function [seatOrder,idealOrder] = airseatSetupSim(boardingType, simType, numRows, numSeatsPerRow, ...
                                     crossZoneFactor, expRndMeanStowTime, ...
                                     expRndMeanGetUpSitTime)
% AIRSEATSETUPSIM Setup simulation options for airplane seating model.
%    Usage: 
%        airseatSetupSim('boardingType, simType)
%
%
%    The boardingType is one of the following:
%    random: Random throughout the aircraft
%    backToFrontZone: Zone based boarding from rear to front
%    frontToBackZone:  Zone based boarding from front to rear
%    alternateRowZone: Zone based boarding from rear to front but
%    zones have alternate rows in them 
%    windowToAisleZone: Zone based boarding with window seats all in
%    zone 1, middle seats in zone 2, and so on. 
%
%    The simType option controls repeatability of simulations.
%    The 'repeatable' simulation type option ensures that all 
%    boarding options are evaluated on a level playing field with the 
%    same times taken by a particular passenger to stow bags and sit 
%    down. Additionally, a passenger will also take the same time in
%    all options to get up and reseat themselves when they need to 
%    let another passenger get access to their seat. Set the simulation 
%    type option to 'resetRepeatable' if you want to reset to a new
%    set of random values for stowage and reseat times.
%    
    % Generally works only for even number of rows and even number of seats per row
    assert(numRows/2 == round(numRows/2));
    assert(numSeatsPerRow/2 == round(numSeatsPerRow/2));
    
    if strcmp(simType, 'repeatable')
        % Load RNG state to make simulations repeatable
        load airseatRndSeed
        rng(asRndSeedState); %#ok
    else 
        % Reset random seed 
        assert(strcmp(simType, 'resetRepeatable'));
        asRndSeedState = rng('shuffle');
        save airseatRndSeed asRndSeedState;
    end
    
    % Setup seating and reseating times for each passenger
    stowTimes = exprnd(expRndMeanStowTime, 1, numRows*numSeatsPerRow) + ...
        expRndMeanStowTime;
    stowTimes(stowTimes > 5*expRndMeanStowTime) = 5*expRndMeanStowTime;
    getUpSitTimes = exprnd(expRndMeanGetUpSitTime, 1, numRows*numSeatsPerRow) + ...
        expRndMeanGetUpSitTime;
    getUpSitTimes(getUpSitTimes > 4*expRndMeanGetUpSitTime) = ...
        4*expRndMeanGetUpSitTime;

    % Derive seat ordering based on boarding option specified
    [seatOrder,idealOrder] = LocGenSeatingOrder(boardingType, numRows, numSeatsPerRow, ...
                                   stowTimes, getUpSitTimes, crossZoneFactor);
end

function [seatOrder,idealOrder] = LocGenSeatingOrder(orderType, numRows, numSeats, stowTimes, getUpSitTimes, crossZoneFactor)

    seatOrder.rows = [];
    seatOrder.seats = [];
    seatOrder.stowageTimes = [];
    seatOrder.getUpAndSitTimes = [];
     
    if strcmp(orderType, 'random')
        % Random order throughout aircraft
        weights = ones(numRows, numSeats);
        idealWeights = weights;
    elseif strcmp(orderType, 'backToFrontZone') || ...
            strcmp(orderType, 'frontToBackZone')
        % Weighted back to front to simulate real world back to
        % front seating. Even though passengers are called by 'zone',
        % there is some mixture between zones because of proirity seating
        % and late arriving passengers.
        numZones = 4;

        rowsPerZone = numRows/numZones;
        weights = [];
        for k = 1:numZones
            locWeights = ones(1, numSeats) * (crossZoneFactor^(k-1));
            locWeights = locWeights(ones(1, rowsPerZone),:);
            weights = [weights; locWeights]; %#ok
        end
        if strcmp(orderType, 'frontToBackZone')
            weights = flipud(weights);
            idealWeights = reshape(1:(numSeats*numRows), numSeats, numRows)';
            idealWeights = [idealWeights(:,1:numSeats/2), ...
                            fliplr(idealWeights(:,(numSeats/2+1):numSeats))];
        else
            idealWeights = reshape((numSeats*numRows):-1:1, numSeats, numRows)';
            idealWeights = [fliplr(idealWeights(:,(numSeats/2+1):numSeats)), ...
                           idealWeights(:,1:numSeats/2)];
        end
    elseif strcmp(orderType, 'alternateRowZone')
        numZones = 4;
        rowsPerZone = numRows/numZones;
        weights = nan(numRows/2,numSeats*2);
        for k = 1:numZones
            locWeights = ones(1, numSeats) * (crossZoneFactor^(k-1));
            locWeights = locWeights(ones(1, rowsPerZone),:);
            if k <= numZones/2
                kIdx = (k-1)*rowsPerZone;
                weights((kIdx+1):(kIdx+rowsPerZone),1:numSeats) = locWeights;
            else
                kIdx = (k-numZones/2-1)*rowsPerZone;
                weights((kIdx+1):(kIdx+rowsPerZone),...
                        (numSeats+1):numSeats*2) = locWeights;
            end
        end
        weights = reshape(weights', numSeats, numRows)';
        
        idealWeights = reshape(1:(numSeats*numRows/2), numSeats, numRows/2);
        idealWeights = [idealWeights; idealWeights+(numSeats*numRows/2)];
        idealWeights = reshape(idealWeights, numSeats, numRows)';
        idealWeights = flipud([idealWeights(:,1:numSeats/2), ...
                            fliplr(idealWeights(:,(numSeats/2+1):numSeats))]);
    elseif (strcmp(orderType, 'windowToAisleZone'));
        % Give all people at windows first priority followed by
        % those in middle, followed by those in aisle
        assert(strcmp(orderType, 'windowToAisleZone'));
        weights = crossZoneFactor.^(0:(numSeats/2-1));
        weights = [fliplr(weights) weights];
        weights = weights(ones(1, numRows),:);
        
        idealWeightsRow = (1:2:(2*numRows))';
        idealWeights = zeros(numRows, numSeats);
        offset = 0;
        for s = 1:(numSeats/2)
            idealWeights(:,s) = idealWeightsRow+offset;
            idealWeights(:,numSeats-s+1) = idealWeightsRow+offset+1;
            offset = offset + numRows*2;
        end
        idealWeights = flipud(idealWeights);
    else
        % Give all people at windows first priority followed by
        % those in middle, followed by those in aisle and also
        % go alternate rows
        assert(strcmp(orderType, 'windowToAisleAlternateZone'));
        
        weights = [1 crossZoneFactor^2]';
        weights = weights(:, ones(1, numRows/4));
        weights = weights(:);
        weights = [weights; crossZoneFactor*weights];
        weights = [weights*crossZoneFactor^8 weights*crossZoneFactor^4 weights];
        weights = [weights, fliplr(weights)];
        
        idealWeights = LocGetIdealOrder(weights);

        % Do first set of alternate rows
        % weights = [1 crossZoneFactor]';
        % weights = weights(:,ones(1, numRows/4));
        % weights = weights(:);
        % weights = [weights; (crossZoneFactor^2)*weights];
        % weights = [weights*crossZoneFactor^8 weights*crossZoneFactor^4 weights];
        % weights = [weights, fliplr(weights)];

    end
    assert(size(weights,1) == numRows);
    assert(size(weights,2) == numSeats);
    seatOrder.weights = weights;
    
    weights = weights'; weights = weights(:)';

    seatOrder.seatRaw = datasample(1:numRows*numSeats, numRows*numSeats, ...
                         'replace', false, 'weights', weights);
    seatOrder.rows = ceil(seatOrder.seatRaw/numSeats);
    seatOrder.seats = rem(seatOrder.seatRaw,numSeats);
    seatOrder.seats(seatOrder.seats == 0) = numSeats;
    seatOrder.seatLetters = char(seatOrder.seats+64);
    seatOrder.stowageTimes = stowTimes(seatOrder.seatRaw);
    seatOrder.getUpAndSitTimes = getUpSitTimes(seatOrder.seatRaw);    
    
    idealOrder.weights = idealWeights;
    if strcmp(orderType, 'random')
        idealOrder.seatRaw = seatOrder.seatRaw;
    else
        idealOrder.seatRaw = idealOrder.weights'; 
        idealOrder.seatRaw = idealOrder.seatRaw(:)';
    end
    idealOrder.rows = ceil(idealOrder.seatRaw/numSeats);
    idealOrder.seats = rem(idealOrder.seatRaw,numSeats);
    idealOrder.seats(idealOrder.seats == 0) = numSeats;
    idealOrder.seatLetters = char(idealOrder.seats+64);
    idealOrder.stowageTimes = stowTimes(idealOrder.seatRaw);
    idealOrder.getUpAndSitTimes = getUpSitTimes(idealOrder.seatRaw);

end

function idealOrder = LocGetIdealOrder(weights)
    unW = unique(weights);
    unW = sort(unW, 'descend');
    
    nRows = size(weights, 1); nSeats = size(weights, 2);
    weightsBackToFront = fliplr(weights');
    weightsBackToFront = weightsBackToFront(:);
    
    nUsed = 0;
    idealOrder = weightsBackToFront;
    for k = 1:length(unW)
        nUnWk = weightsBackToFront == unW(k);
        idealOrder(nUnWk) = (1:nnz(nUnWk)) + nUsed;
        nUsed = nUsed + nnz(nUnWk);
    end
    idealOrder = reshape(idealOrder, nSeats, nRows);
    idealOrder = fliplr(idealOrder)';
end
