classdef airseatViz < simevents.CustomObserverInterface
% AIRSEATVIZ Helper class that draws results of airplane seating model.
%
%
    properties (Access=private)
        mFig;
        mPatches = cell(1, 20);
        
        mStatusTxt = 0;
        
        mNumRows;
        mNumSeats;

        mDeleted = 0;
    end
    
    % ---------------------------------------------------------------------
    methods (Access=public)
        
        function this = airseatViz(numRows, numSeats)
            this.mNumRows = numRows;
            this.mNumSeats = numSeats;
        end
        
        function fig = getFigure(this)
            fig = this.mFig;
        end
        
        function blks = getBlocksToObserve(this) %#ok<MANU>
            blks1 = find_system('airseat', 'FollowLinks', 'on', ...
                                'BlockType', 'SingleServer');
            blks2 = find_system('airseat', 'FollowLinks', 'on', ...
                                'BlockType', 'EntitySink');
            blks = union(blks1,blks2);
        end
        
        function p = getPace(this) %#ok<MANU>
            p = 5;
        end
        
        % ------------------------------------------------------------------
        % Model callbacks
        % ------------------------------------------------------------------
        
        function initialize(this, ~)
            this.mFig = figure;
            grayCol = [0.95 0.95 0.95];
            for k = 1:this.mNumRows
                for m = 1:(this.mNumSeats/2)
                    [x,y] = this.circpatch(k, m);
                    fill(x, y, grayCol);
                    hold on
                    [x,y] = this.circpatch(k, -m);
                    fill(x, y, grayCol);
                end
                t = text(k, (this.mNumSeats/2)+1, sprintf('%d', k));
                set(t, 'horizontalAlignment','center')
                set(t, 'fontsize', 7);
                set(t, 'fontweight', 'bold');

                t = text(k, -((this.mNumSeats/2)+1), sprintf('%d', k));
                set(t, 'horizontalAlignment','center')
                set(t, 'fontsize', 7);
                set(t, 'fontweight', 'bold');
            end
            cVal = 1;
            for k = -(this.mNumSeats/2):(this.mNumSeats/2)
                if k ~= 0
                    t = text(0, k, sprintf('%s', char(64+cVal)));
                    set(t, 'horizontalAlignment','center')
                    set(t, 'fontsize', 7);
                    set(t, 'fontweight', 'bold');
                    cVal = cVal + 1;
                end
            end
            set(gca,'dataAspectRatio',[10 10 1]);
            set(gca, 'ytick', []);
            set(gca, 'xtick', []);
            set(gca, 'ylim', [-((this.mNumSeats/2)+2) ((this.mNumSeats/2)+2)])
            set(gca, 'xlim', [-1 this.mNumRows+2]);
            
            this.mStatusTxt = text(11, -this.mNumSeats-2, '');
            set(this.mStatusTxt, 'horizontalAlignment','center')
            set(this.mStatusTxt, 'fontsize', 10)
            set(this.mStatusTxt, 'fontweight', 'bold')
        end
        
        % ------------------------------------------------------------------
        % Entity callbacks
        % ------------------------------------------------------------------
        
        function entityAdvance(this, entity, ~, to)
            enStruct = sedb.eninfo(entity);
            if strcmp(get_param(to, 'blocktype'), 'SingleServer')
                prevRow = enStruct.Attributes.CurrentSeatLoc-1;
                if prevRow > 0 && ~isempty(this.mPatches{prevRow})
                    %disp(['Deleting ' num2str(prevRow)])
                    delete(this.mPatches{prevRow}.reg);
                    delete(this.mPatches{prevRow}.txt);
                    this.mPatches{prevRow} = [];
                end
                currRow = enStruct.Attributes.CurrentSeatLoc;
                destRow = enStruct.Attributes.Row;
                
                [x,y] = this.circpatch(currRow, 0);
                p.reg = fill(x, y, 'r');
                t = text(currRow, 0, sprintf('%d', destRow));
                set(t, 'horizontalAlignment','center')
                set(t, 'fontsize', 7);
                set(t, 'fontweight', 'bold');
                p.txt = t;
                this.mPatches{currRow} = p;
                %disp(['Adding ' num2str(currRow)])
            end
        end
        
        function entityDestroy(this, entity, ~)
            enStruct = sedb.eninfo(entity);
            currRow = enStruct.Attributes.CurrentSeatLoc;
            if ~isempty(this.mPatches{currRow})
                delete(this.mPatches{currRow}.reg);
                delete(this.mPatches{currRow}.txt);
                this.mPatches{currRow} = [];
                %disp(['Deleting ' num2str(currRow)])
            end
            currSeat = enStruct.Attributes.Seat;
            if currSeat > (this.mNumSeats/2)
                currSeatMod = currSeat - (this.mNumSeats/2);
            else
                currSeatMod = currSeat - (this.mNumSeats/2 + 1);
            end
            this.mDeleted = this.mDeleted + 1;
            if this.mDeleted == (this.mNumSeats * this.mNumRows)
                assignin('base', 'asLastSeatingTime', sedb.simtime);
            end
            
            [x,y] = this.circpatch(currRow, currSeatMod);
            fill(x, y, [(1-this.mDeleted/(this.mNumSeats*this.mNumRows)) 1 this.mDeleted/(this.mNumSeats*this.mNumRows)]);

            passSeat = sprintf('%d%s', currRow, char(64 + currSeat));
            
            set(this.mStatusTxt, 'String', ...
                              sprintf('Seating %s [%d] at %4.2f mins\n', ...
                                      passSeat, ...
                                      this.mDeleted, ...
                                      sedb.simtime));
        end
        
    end
    methods(Static)
        function [x,y] = circpatch(k,m)
            r = 0.35;
            x = -(r+eps):0.01:(r+eps);
            y = sqrt(r^2 - x.^2);
            
            x(~isreal(y)) = [];
            y(~isreal(y)) = [];
            x = [-r x r];
            y = [0 y 0];
            
            x = [x fliplr(x)];
            y = [y -fliplr(y)];
            x = x + k;
            y = y + m;
        end    
    end % methods    
end % classdef
